<?
require_once("DbUser.php");

require_once("exceptions/DbUserException.php");
require_once("exceptions/DbException.php");
require_once("exceptions/AccessException.php");


class Access {
	
	private $requesturi = '';
	private $status = true;
	private $message = '';
	
	/**
	 * Constructor
	 * @param string $level either "user" or "admin"
	 * @throws DbUserException if the user could not be authenticated
	 * @throws AccessException if $level is "admin" but logged in user is not
	 * administrator
	 * @throws DbAdminException if admin could not be authenticated
	 * @throws DbException if database problem
	 */
	public function __construct($level) {
		Logger::write("Entering Access::__construct", 1);
		
		if(isset($_REQUEST["login"])) {
			$username = $_REQUEST["username"];
			$password = $_REQUEST["password"];
			$dbuser = DbUser::getAuthUser($username, $password);
			if($level == "admin" && !$dbuser->isAdmin())
				throw new AccessException("Access::__construct", "Logged in user is not administrator");
			$_SESSION["currentuserid"] = $dbuser->getUserId();
			$this->status = true;
			$this->message = "Successfully logged in ".$dbuser->getUsername();
			return;
		} else {			
			$dbuser = Access::getCurrentUser();
			if($dbuser) {
				if($level == "admin" && $dbuser->isAdmin())
					return;
				elseif($level == "user")
					return;
				else {
					$location = "login.php?request_uri=".urlencode($_SERVER["REQUEST_URI"])."&login=$level";
					header("Location: $location");
				}
			}				
		}
	}
	
	//*************************************************************************
	// Static functions
	//*************************************************************************

	private static $currentuser = null;

	/**
	 * Get the logged in user
	 * @return DbUser the logged in user as a DbUser object or null if no one is
	 * logged in
	 * @throws DbUserException if the user cannot be fetched
	 * @throws DbException if database problem
	 */
	public static function getCurrentUser() {
	 	Logger::write("Entering Access::getCurrentUser", 1);	
	 	if(Access::$currentuser == null) {
	 		if($userid = $_SESSION["currentuserid"]) {
 				try {
 					Access::$currentuser = DbUser::getDbUser($userid);
 				} catch (DbUserException $due) {
 					unset($_SESSION["currentuserid"]);
 					Access::$currentuser = null;
 				}
	 		}
	 	} else {
	 		Logger::write("\tCurrent user: nobody", 1);
	 	}
	 	return Access::$currentuser;
	}
	 
	/**
	 * Logout the currently logged in user
	 * @throws DbUserException if the user cannot be fetched
	 * @throws DbException if database problem
	 */
	public static function logoutUser() {
		Logger::write("Entering Access::logoutUser", 1);
 		$dbuser = Access::getCurrentUser();
		unset($_SESSION["currentuserid"]);	
	 	Access::$currentuser = null;
	}
	
	/**
	 * Check whether the current user, if any, is administrator or not
	 * @return boolean true if current user is admin, false otherwise
	 * @throws DbUserException if the user cannot be fetched
	 * @throws DbException if database problem
	 */
	public static function isCurrentUserAdmin() {
		Logger::write("Entering Access::isCurrentUserAdmin", 1);
		$dbuser = Access::getCurrentUser();
		if($dbuser == null)
			return false;
		return $dbuser->isAdmin();	
	}
	
	/**
	 * Check whether the current user, if any, can see private images or
	 * not
	 * @return boolean true if current user can see private images, false
	 * otherwise
	 * @throws DbUserException if the user cannot be fetched
	 * @throws DbException if database problem
	 */
	public static function isCurrentUserPrivate() {
		Logger::write("Entering Access::isCurrentUserAdmin", 1);
		$dbuser = Access::getCurrentUser();
		if($dbuser == null)
			return false;
		return $dbuser->isPrivate();	
	}
}
