<?
require_once("UniqueId.php");
require_once("Image.php");
require_once("DbGallery.php");
require_once("DbImage.php");
require_once("DbUser.php");
require_once("DbComment.php");

require_once("exceptions/DbException.php");
require_once("exceptions/ActionException.php");
require_once("exceptions/ImageException.php");
require_once("exceptions/DbGalleryException.php");
require_once("exceptions/DbImageException.php");
require_once("exceptions/DbUserException.php");
require_once("exceptions/DbCommentException.php");

class Action {
	
	private $status = true;
	private $message = '&nbsp;';

	public function __construct() {
		Logger::write("Entering Action::__construct", 1);
		$action = $_REQUEST["action"];
		Logger::write("\taction: $action", 1);
		if((bool)$action) {
			switch($action) {
				case "gallery_add":
					return $this->addGallery();
				case "gallery_remove":
					return $this->removeGallery();
				case "gallery_edit":
					return $this->editGallery();
				case "image_add":
					return $this->addImage();
				case "image_remove":
					return $this->removeImage();
				case "image_edit":
					return $this->editImage();
				case "image_move":
					return $this->moveImage();
				case "user_add":
					return $this->addUser();
				case "user_remove":
					return $this->removeUser();
				case "user_edit":
					return $this->editUser();
				case "user_logout":
					return $this->logoutUser();
				case "admin_logout":
					return $this->logoutAdmin();
				case "private_show":
					return $this->showPrivate();
				case "private_hide":
					return $this->hidePrivate();
				case "comment_add":
					return $this->addComment();
				case "comment_remove":
					return $this->removeComment();
				case "image_info_show":
					return $this->showImageInfo();
				case "image_info_hide":
					return $this->hideImageInfo();
			}		
		}
	}
	
	public function getStatus() {
		Logger::write("Entering Action::getStatus", 1);
		return $this->status;
	}
	
	public function getMessage() {
		Logger::write("Entering Action::getMessage", 1);
		return $this->message;
	}
	
	public function addGallery() {
		Logger::write("Entering Action::addGallery", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($galleryname = $_REQUEST["galleryname"]))
			throw new ActionException("Action::addGallery", "Gallery name is blank");
		$isprivate = (bool)$_REQUEST["isprivate"];
		
		$dbGallery = DbGallery::createDbGallery($galleryname, $isprivate);
		$this->message = "Successfully added new gallery: \"".$dbGallery->getName()."\"";
		$this->status = true;
	}
	
	public function removeGallery() {
		Logger::write("Entering Action::removeGallery", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($galleryid = $_REQUEST["galleryid"]))
			throw new ActionException("Action::removeGallery", "No gallery selected");
		$dbimages = DbImage::getAllDbImages($galleryid, true);
		for($i=0; $i<count($dbimages); $i++) {
			$filename = $dbimages[$i]->getFileName();
			$filepath = IMAGES_PATH.DIRECTORY_SEPARATOR.$filename;
			$image = Image::getImage($filepath);
			$image->destroyImageResource();
			$image->deleteImage();	
		}
		DbImage::removeAllDbImages($galleryid);
		$dbgallery = DbGallery::getDbGallery($galleryid);
		DbGallery::removeDbGallery($galleryid);
		$this->message = "Successfully removed gallery \"".$dbgallery->getName()."\"";
		$this->status = true;
	}
	
	public function editGallery() {
		Logger::write("Entering Action::editGallery", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($galleryid = $_REQUEST["galleryid"]))
			throw new ActionException("Action::editGallery", "No gallery selected");
		$galleryname = $_REQUEST["galleryname"];
		$isprivate = $_REQUEST["isprivate"];
		$date = $_REQUEST["gallerydate"];

		$dbgallery = DbGallery::getDbGallery($galleryid);
		$dbgallery->setName($galleryname);
		$dbgallery->setPrivate($isprivate);
		$dbgallery->setDate($date);
		$dbimages = DbImage::getAllDbImages($galleryid, true);
		foreach($dbimages as $dbimage)
			$dbimage->setPrivate($isprivate);
		$this->message = "Successfully updated gallery \"".$dbgallery->getName()."\"";
		$this->status = true;
	}
	
	
	public function addImage() {
		Logger::write("Entering Action::addImage", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		$imagename = $_FILES['file']['name'];
		if(strtolower(array_pop($tmparr = explode('.', $imagename))) == "zip")
			return $this->addZippedImages();
		$filesize = $_FILES['file']['size'];
		if(!($galleryid = $_REQUEST["galleryid"]))
			throw new ActionException("Action::addImage", "No gallery selected");
		$rotation = $_REQUEST["rotation"];
		$isprivate = $_REQUEST["isprivate"];
		$defaultsize = $_REQUEST["defaultsize"];
		$comment = $_REQUEST["comment"];

		$image = Image::createImage($_FILES['file']['tmp_name'], $rotation);
		$dbimage = DbImage::createDbImage($imagename, $galleryid, $image->getDate(), $comment, 
										  $isprivate, $defaultsize, $image->getFileName(), $rotation);
		$this->message = "Successfully added new image \"".$dbimage->getName()."\"";
		$this->status = true;
	}
	
	public function addZippedImages() {
		Logger::write("Entering Action::addZippedImages", 1);
		require_once(SITE_ROOT_DIR.DIRECTORY_SEPARATOR."includes".DIRECTORY_SEPARATOR."pclzip".DIRECTORY_SEPARATOR."pclzip.lib.php");
		$tempdir = GALLERY_ROOT_DIR.DIRECTORY_SEPARATOR.uniqid("");
		if(!mkdir($tempdir))
			throw new ActionException("Action::addZippedImages", "Could not create temporary directory");
		$zipfile = new PclZip($_FILES['file']['tmp_name']);
		if(!($galleryid = $_REQUEST["galleryid"]))
			throw new ActionException("Action::addZippedImages", "No gallery selected");
		$isprivate = $_REQUEST["isprivate"];
		$defaultsize = $_REQUEST["defaultsize"];
		$rotation = $_REQUEST["rotation"];
		$comment = $_REQUEST["comment"];
		$filelist = $zipfile->extract(PCLZIP_OPT_PATH, $tempdir, PCLZIP_OPT_REMOVE_ALL_PATH);
		if($filelist == 0) {
			rmdir($tempdir);
			throw new ActionException("Action::addZippedImages", "Could not extract zip file\n".$zipfile->errorInfo(true));
		}
		$filecount = count($filelist);
		foreach($filelist as $file) {
			$filepath = $file["filename"];
			$imagename = basename($filepath);
			$image = Image::createImage($filepath, $rotation);
			$dbimage = DbImage::createDbImage($imagename, $galleryid, $image->getDate(), $comment, 
											  $isprivate, $defaultsize, $image->getFileName());
		}
		if(!rmdir($tempdir))
			throw new ActionException("Action::addZippedImages", "Could not remove temporary directory");
		$this->message = "Successfully added ".$filecount." zipped images to gallery";
		$this->status = true;
	}
	
	public function removeImage() {
		Logger::write("Entering Action::removeImage", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($imageid = $_REQUEST["imageid"]))
			throw new ActionException("Action::removeImage", "No image selected");
		$dbimage = DbImage::getDbImage($imageid);	
		$image = Image::getImage(IMAGES_PATH.DIRECTORY_SEPARATOR.$dbimage->getFilename());
		$image->destroyImageResource();
		$image->deleteImage();
		DbImage::removeDbImage($imageid);
		$this->message = "Successfully removed image \"".$dbimage->getName()."\"";
		$this->status = true;
	}
	
	public function editImage() {
		Logger::write("Entering Action::editImage", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($imageid = $_REQUEST["imageid"]))
			throw new ActionException("Action::editImage", "No image selected");
		$comment = $_REQUEST["comment"];
		$isprivate = $_REQUEST["isprivate"];
		$defaultsize = $_REQUEST["defaultsize"];
		$rotation = $_REQUEST["rotation"];
		$date = $_REQUEST["date"];

		$dbimage = DbImage::getDbImage($imageid);
		$dbimage->setComment($comment);
		$dbimage->setPrivate($isprivate);
		$dbimage->setDefaultSize($defaultsize);
		$dbimage->setDate($date);

		$oldrotation = $dbimage->getRotation();
		if($oldrotation != $rotation) {
			$dbimage->setRotation($rotation);
			$image = Image::getImage(IMAGES_PATH.DIRECTORY_SEPARATOR.$dbimage->getFileName());
			$image->rotateImage($rotation);
			$thumbfilepath = THUMBS_PATH.DIRECTORY_SEPARATOR.$dbimage->getFileName(); 
			unlink($thumbfilepath);
			$image->createThumbNail(THUMBS_WIDTH, THUMBS_HEIGHT, $thumbfilepath);
		}
		$this->message = "Successfully updated image \"".$dbimage->getName()."\"";
		$this->status = true;
	}
	
	public function moveImage() {
		Logger::write("Entering Action::moveImage", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($imageid = $_REQUEST["imageid"]))
			throw new ActionException("Action::moveImage", "No image selected");
		if(!($togalleryid = $_REQUEST["togalleryid"]))
			throw new ActionException("Action::moveImage", "No destination gallery selected");

		$dbimage = DbImage::getDbImage($imageid);
		$dbimage->setGalleryId($togalleryid);
		$dbgallery = DbGallery::getDbGallery($togalleryid);
		$this->message = "Successfully moved image \"".$dbimage->getName().
						 "\" to gallery \"".$dbgallery->getName()."\"";
		$this->status = true;
	}
	
	public function addUser() {
		Logger::write("Entering Action::addUser", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($username = $_REQUEST["username"]))
			throw new ActionException("Action::addUser", "Username is blank");
		if(!($password = $_REQUEST["password"]))
			throw new ActionException("Action::addUser", "Password is blank");
		$isprivate = isset($_REQUEST["isprivate"])?$_REQUEST["isprivate"]:0;
		$isadmin = isset($_REQUEST["isadmin"])?$_REQUEST["isadmin"]:0;
		if(!($email = $_REQUEST["email"]))
			throw new ActionException("Action::addUser", "E-mail is blank");

		$dbuser = DbUser::createDbUser($username, $password, $isprivate, $email, $isadmin);
		$this->message = "Successfully added new user \"".$dbuser->getUsername()."\"";
		$this->status = true;
	}
	
	public function removeUser() {
		Logger::write("Entering Action::removeUser", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($userid = $_REQUEST["userid"]))
			throw new ActionException("Action::removeUser", "No user selected");

		$dbuser = DbUser::getDbUser($userid);
		DbUser::removeDbUser($userid);
		$this->message = "Successfully removed user \"".$dbuser->getUsername()."\"";
		$this->status = true;
	}
	
	public function editUser() {
		Logger::write("Entering Action::editUser", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($userid = $_REQUEST["userid"]))
			throw new ActionException("Action::editUser", "No user selected");
		$newpassword = $_REQUEST["newpassword"];
		$oldpassword = $_REQUEST["oldpassword"];
		$isprivate = $_REQUEST["isprivate"];
		$isadmin = $_REQUEST["isadmin"];
		$email = $_REQUEST["email"];

		$dbuser = DbUser::getDbUser($userid);
		if((bool)$newpassword)
			$dbuser->setPassword($oldpassword, $newpassword);
		$dbuser->setPrivate($isprivate);
		$dbuser->setEmail($email);
		$dbuser->setAdmin($isadmin);
		$this->message = "Successfully updated user \"".$dbuser->getUsername()."\"";
		$this->status = true;
	}
	
	public function logoutUser() {
		Logger::write("Entering Action::logoutUser", 1);
		Access::logoutUser();
	} 
	
	public function logoutAdmin() {
		Logger::write("Entering Action::logoutAdmin", 1);
		Access::logoutAdmin();
		$this->message = "Successfully logged out admin";
		$this->status = true;
	}
	
	public function showPrivate() {
		Logger::write("Entering Action::showPrivate", 1);
		if($_SESSION["adminloggedin"]) {
			$_SESSION["showprivate"] = true;
		} else { 
			$dbuser = Access::getCurrentUser();
			if($dbuser != null && $dbuser->isPrivate()) {
				$_SESSION["showprivate"] = true;
			} else {
				throw new ActionException("Action::showPrivate", "You are not authorized to see private images");
			}
			$this->message = "Successfully changed privacy status to \"show\"";
			$this->status = true;
		}
	}
	
	public function hidePrivate() {
		Logger::write("Entering Action::hidePrivate", 1);
		unset($_SESSION["showprivate"]);
		$this->message = "Successfully changed privacy status to \"hide\"";
		$this->status = true;
	}
	
	public function addComment() {
		Logger::write("Entering Action::addComment", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($comment = $_REQUEST["comment"]))
			throw new ActionException("Action::addComment", "Comment is blank");
		$userid = $_REQUEST["userid"];
		$imageid = $_REQUEST["imageid"];
		$dbcomment = DbComment::createDbComment($userid, $comment, $imageid);
		$this->message = "Successfully added new comment";
		$this->status = true;
	}
	
	public function removeComment() {
		Logger::write("Entering Action::removeComment", 1);
		$uniqueid = $_REQUEST["uniqueid"];
		if($uniqueid && UniqueId::isUnique($uniqueid))
			UniqueId::setUnique($uniqueid);
		else
			return;
		if(!($commentid = $_REQUEST["commentid"]))
			throw new ActionException("Action::removeComment", "Comment id is blank");
		DbComment::removeDbComment($commentid);
		$this->message = "Successfully removed comment";
		$this->status = true;
	}

	public function showImageInfo() {
		Logger::write("Entering Action::showImageInfo", 1);
		$_SESSION["showimageinfo"] = true;
	}
	
	public function hideImageInfo() {
		Logger::write("Entering Action::hideImageInfo()", 1);
		unset($_SESSION["showimageinfo"]);
	}
}
?>