<?
class DbComment {
	private $commentid = 0;
	private $userid = 0;
	private $comment = '';
	private $date = '';
	private $peer = '';
	private $imageid = 0;
	
	/**
	 * Constructor. Creates a new DbComment object representing a comment record
	 * in the database.
	 * @param integer $commentid the unique id
	 * @param integer $userid the id of the user that has made this comment
	 * @param string $comment the comment
	 * @param string $date the date the comment was submitted
	 * @param string $peer the ip number of the host used to make the comment
	 * @param integer $imageid the id of the image this comment relates to
	 */
	public function __construct($commentid, $userid, $comment, $date, $peer, $imageid) {
		Logger::write("Entering DbComment::__construct", 1);
		$this->commentid = $commentid;
		$this->userid = $userid;
		$this->comment = $comment;
		$this->date = $date;
		$this->peer = $peer;
		$this->imageid = $imageid;
	}
	
	/**
	 * Get the unique id of this comment
	 * @return integer the id of this comment
	 */
	public function getCommentId() {
		Logger::write("Entering DbComment::getCommentId", 1);
		return $this->commentid;
	}
	
	/**
	 * Get the id of the user that submitted this comment
	 * @return integer the id of the user that submitted this comment
	 */
	public function getUserId() {
		Logger::write("Entering DbComment::getUserId", 1);
		return $this->userid;
	}
	
	/**
	 * Get the comment
	 * @return string the comment
	 */
	public function getComment() {
		Logger::write("Entering DbComment::getComment", 1);
		return $this->comment;
	}
	
	/**
	 * Set the comment
	 * @param string $comment the comment to set
	 * @throws DbException if database problem
	 */
	public function setComment($comment) {
		Logger::write("Entering DbComment::setComment", 1);
		if($this->comment == $comment)
			return;
		$peer = $_SERVER['REMOTE_ADDR'];
		$dbconn = Db::getInstance();
		$query = "UPDATE comments ".
				 "SET comment = '$comment', date = NOW(), peer = '$peer' ".
				 "WHERE id = ".$this->commentid;
		$dbconn->update($query);
		$this->comment = $comment;
	}
	
	/**
	 * Get the date when this comment was submitted
	 * @return string the date of this comment
	 */
	public function getDate() {
		Logger::write("Entering DbComment::getDate", 1);
		return $this->date;
	}
	
	/**
	 * Get the ip number of the host used to submit this comment
	 * @return string the ip number of the submitter
	 */
	public function getPeer() {
		Logger::write("Entering DbComment::getPeer", 1);
		return $this->peer;
	}
	
	/**
	 * Get the id of the image this comment relates to
	 * @return integer the image id of this comment
	 */
	public function getImageId() {
		Logger::write("Entering DbComment::getImageId", 1);
		return $this->imageid;
	}
	
	//************************************************************************
	// Static functions
	//************************************************************************
	
	/**
	 * Create a new DbComment and insert it into the database
	 * @param integer $userid the unique id of the user that submits the comment
	 * @param string $comment the comment submitted
	 * @param integer $imageid the unique id of the image commented
	 * @throws DbException if database problem
	 */
	public static function createDbComment($userid, $comment, $imageid) {
		Logger::write("Entering DbComment::createDbComment", 1);
		$peer = $_SERVER["REMOTE_ADDR"];
		$dbconn = Db::getInstance();
		$query = "INSERT INTO comments (user_id, comment, date, peer, image_id) ".
				 "VALUES ($userid, '$comment', NOW(), '$peer', $imageid)";
		$result = $dbconn->insertQueryLast($query, "comments");
		$line = $dbconn->fetchAssoc($result);
		$dbcomment = new DbComment($line["id"], $line["user_id"], $line["comment"], $line["date"], $line["peer"], $line["image_id"]);
		return $dbcomment;
	}
	
	/**
	 * Get the comment with the specified id as an DbComment object
	 * @param integer $commentid the id of the comment to fetch
	 * @return DbComment the comment as a DbComment instance
	 * @throws DbCommentException if the comment does not exist
	 * @throws DbException if database problem
	 */
	public static function getDbComment($commentid) {
		Logger::write("Entering DbComment::getDbComment", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * ".
				 "FROM comments ".
				 "WHERE id = ".$commentid;
		$result = $dbconn->query($query);
		$line = $dbconn->fetchAssoc($result);
		if(!$line)
			throw new DbCommentException("DbComment::getDbComment", "Comment with id ".$commentid." was not found in database");
		$dbcomment = new DbComment($line["id"], $line["user_id"], $line["comment"], $line["date"], $line["peer"], $line["image_id"]);
		return $dbcomment;
	}
	
	/**
	 * Get all submitted comments for a specific image id
	 * @param integer $imageid the image id that the comments should be
	 * connected to
	 * @return array an array of DbComment objects or null if no exist
	 * @throws DbException if database problem
	 */
	public static function getAllDbComments($imageid) {
		Logger::write("Entering DbComment::getAllDbComments", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * ".
				 "FROM comments ".
				 "WHERE image_id = $imageid";
		$result = $dbconn->query($query);
		$dbcomments = array();
		while($line = $dbconn->fetchAssoc($result))
			$dbcomments[] = new DbComment($line["id"], $line["user_id"], $line["comment"], $line["date"], $line["peer"], $line["image_id"]);
		return $dbcomments;
	}
	
	/**
	 * Remove a comment from the database
	 * @param integer $commentid the id of the comment to be removed
	 * @throws DbException if database problem
	 */		
	public static function removeDbComment($commentid) {
		Logger::write("Entering DbComment::removeDbComment", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM comments WHERE id = $commentid";
		$dbconn->update($query);
	}
	
	/**
	 * Remove all comments from database belonging to a specific image
	 * @param integer $imageid the id of the image of which all comments should
	 * be removed
	 * @throws DbException if database problem
	 */
	public static function removeAllDbComments($imageid) {
		Logger::write("Entering DbComment::removeAllDbComments", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM comments WHERE image_id = $imageid";
		$dbconn->update($query);
	}
		
	/**
	 * Remove all comments from database belonging to a specific user
	 * @param integer $userid the id of the user of which all comments should be
	 * removed
	 * @throws DbException if database problem
	 */
	public static function removeAllDbCommentsByUser($userid) {
		Logger::write("Entering DbComment::removeAllDbCommentsByUser", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM comments WHERE user_id = $userid";
		$dbconn->update($query);
	}
}


?>