<?php
/**
 * This class represent a database record for a user. It also implements static
 * functions for getting, creating and removing users
 */
class DbUser {
	private $userid = 0;
	private $username = '';
	private $password = '';
	private $isprivate = false;
	private $email = '';
	private $peer = '';
	private $isadmin = false;
	
	/**
	 * Constructor.
	 * @param integer $id the id of the database record
	 * @param string $username the username
	 * @param bool $isprivate flag whether this user can see private images and
	 * galleries or not
	 * @param string $email the email
	 * @param string $peer the ip addess of the user when registering
	 */
	public function __construct($userid, $username, $password, $isprivate, $email, $peer, $isadmin) {
	 	Logger::write("Entering DbUser::__construct", 1);	
		$this->userid = $userid;
		$this->username = $username;
		$this->password = $password;
		$this->email = $email;
		$this->isprivate = (bool)$isprivate;
		$this->peer = $peer;
		$this->isadmin = $isadmin;	
	}
	
	/**
	 * Get the id of this user record
	 * @return integer the id of this user
	 */
	public function getUserId() {
	 	Logger::write("Entering DbUser::getUserId", 1);	
		return $this->userid;
	}
	
	/**
	 * Get the username of this user record
	 * @return string the username
	 */
	public function getUsername() {
	 	Logger::write("Entering DbUser::getUsername", 1);	
		return $this->username;
	}
	
	/**
	 * Get the ip address this user registered from
	 * @return string ip address this user registered from
	 */
	public function getPeer() {
	 	Logger::write("Entering DbUser::getPeer", 1);	
		return $this->peer;
	}

	/**
	 * Set a new password for this user
	 * @param string $oldpassword the old password (for verification)
	 * @param string $newpassword the new password
	 * @throws DbException if database problem
	 * @throws DbUserException if the old password did not authenticate
	 */
	public function setPassword($oldpassword, $newpassword) {
	 	Logger::write("Entering DbUser::setPassword", 1);	
		$dbconn = Db::getInstance();
		$query = "SELECT COUNT(password) ".
				 "FROM users ".
				 "WHERE password = PASSWORD('$oldpassword') ".
				 "AND id = ". $this->userid;
		$result = $dbconn->query($query);
		$line = $dbconn->fetchRow($result);
		if((bool) $line[0])
			throw new DbUserException("DbUser::setPassword", "The old password did not verify towards the database");
		$query = "UPDATE users ".
				 "SET password = PASSWORD('$password') ".
				 "WHERE id = ". $this->userid; 
		$dbconn->update($query);
		$this->password = $password;
	}
	
	/**
	 * Get the e-mail address of this user
	 * @return string the e-mail address of this user
	 */
	public function getEmail() {
	 	Logger::write("Entering DbUser::getEmail", 1);	
		return $this->email;	
	}
	
	/**
	 * Set the e-mail address of this user
	 * @param string $email the new email address of this user
	 * @throws DbUserException if the new e-mail already exists
	 * @throws DbException if database problem
	 */
	public function setEmail($email) {
	 	Logger::write("Entering DbUser::setEmail", 1);	
		if($this->email == $email)
			return;
		if(Db::emailExist($email))
			throw new DbUserException("DbUser::setEmail", "E-mail \"". $email. "\" already exists in database");
		$dbconn = Db::getInstance();
		$query = "UPDATE users ".
				 "SET email = '$email' ".
				 "WHERE id = ". $this->userid; 
		$dbconn->update($query);
		$this->email = $email;
	}
	
	/** 
	 * Check whether this user can see private images
	 * @return bool true id user can see private images, false otherwise
	 */
	public function isPrivate() {
	 	Logger::write("Entering DbUser::isPrivate", 1);	
		return $this->isprivate;
	}
	
	/**
	 * Set whether this user can see private images
	 * @param bool $isprivate true if user should see private images, false
	 * otherwise
	 * @throws DbException if database problem
	 */
	public function setPrivate($isprivate) {
	 	Logger::write("Entering DbUser::setPrivate", 1);	
		if($this->isprivate == $isprivate)
			return;
		if($isprivate)
			$ispriv = 1;
		else
			$ispriv = 0;
		$dbconn = Db::getInstance();
		$query = "UPDATE users ".
				 "SET is_private = $ispriv ".
				 "WHERE id = ". $this->userid; 
		$dbconn->update($query);
		$this->isprivate = $isprivate;
	}
	
	/**
	 * Check whether this user is administrator
	 * @return bool true if user is administrator, false otherwise
	 */
	public function isAdmin() {
		Logger::write("Entering DbUser::isAdmin", 1);
		return $this->isadmin;
	}
	
	/**
	 * Set whether this user is administrator
	 * @param bool $isadmin true if user admin, false otherwise
	 * @throws DbException if database problem
	 */
	public function setAdmin($isadmin) {
	 	Logger::write("Entering DbUser::setAdmin", 1);	
		if($this->isadmin == $isadmin)
			return;
		if($isadmin)
			$isadm = 1;
		else
			$isadm = 0;
		$dbconn = Db::getInstance();
		$query = "UPDATE users ".
				 "SET is_admin = $isadm ".
				 "WHERE id = ". $this->userid; 
		$dbconn->update($query);
		$this->isadmin = $isadmin;
	}
		
	/*******************************************************************
	 * Static functions
	 *******************************************************************/

	/**
	 * Check if the given e-mail address already exists in the database
	 * @return bool true if it exists, false otherwise
	 * @throws DbException if database problem
	 */
	public static function emailExist($email) {
	 	Logger::write("Entering DbUser::emailExist", 1);	
		$dbconn = Db::getInstance();
		$query = "SELECT count(email) FROM users WHERE email = '$email'";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchRow($result);
		return((bool) $line[0]);
	}

	/**
	 * Check if the given username already exists in the database
	 * @return bool true if it exists, false otherwise
	 * @throws DbException if database problem
	 */
	public static function nameExist($username) {
	 	Logger::write("Entering DbUser::nameExist", 1);	
		$dbconn = Db::getInstance();
		$query = "SELECT count(name) FROM users WHERE name = '$username'";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchRow($result);
		return((bool) $line[0]);
	}
	
	/**
	 * Create a new user record in the database
	 * @param string $username the username of the new user
	 * @param string $password the password of the new user
	 * @param bool $isprivate true if user should be able to see private images,
	 * false otherwise
	 * @param string $email the email address of the new user
	 * @return DbUser the newly created user
	 * @throws DbUserException if the new user could not be created
	 * @throws DbException if database problem
	 */
	public static function createDbUser($username, $password, $isprivate, $email, $isadmin = false) {
	 	Logger::write("Entering DbUser::createDbUser", 1);	
		if(DbUser::nameExist($username)) {
			throw new DbUserException("DbUser::createUser", "Username \"". $username. "\" already exists in database");
		} 
		if(DbUser::emailExist($email)) {
			throw new DbUserException("DbUser::createUser", "E-mail \"". $email. "\" already exists in database");
		}
		$dbconn = Db::getInstance();
		
		$peer = $_SERVER['REMOTE_ADDR'];
		$query = "INSERT INTO users (name, password, is_private, email, peer, is_admin) ".
				 "VALUES ('$username', '$password', $isprivate, '$email', '$peer', $isadmin)";
		$result = $dbconn->insertQueryLast($query, "users");
		$line = $dbconn->fetchAssoc($result);
		$dbUser = new DbUser($line["id"], $line["name"], $line["password"], $line["is_private"], $line["email"], $line["peer"], $line["is_admin"]);
		return $dbUser;
	}
	
	/**
	 * Get a user record as a DbUser object
	 * @param integer $userid the id of the user
	 * @throws DbUserException if the user cannot be fetched
	 * @throws DbException if database problem
	 */
	public static function getDbUser($userid) {
	 	Logger::write("Entering DbUser::getDbUser", 1);	
		$dbconn = Db::getInstance();
		$query = "SELECT name, password, is_private, email, peer, is_admin ".
				 "FROM users ".
				 "WHERE id = $userid";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchAssoc($result);
		if(!$line)
			throw new DbUserException("DbUser::getUser", "User with id '". $userid. "' was not found in database");
		$dbUser = new DbUser($userid, $line["name"], $line["password"], $line["is_private"], $line["email"], $line["peer"], $line["is_admin"]);
		return $dbUser;
	}

	/**
	 * Get all users from the database
	 * @return array an array with all DbUser objects
	 * @throws DbException if database problem
	 */
	public static function getAllDbUsers() {
	 	Logger::write("Entering DbUser::getAllDbUsers", 1);	
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM users";
		$result = $dbconn->query($query);
		$dbusers = array();
		while($line = $dbconn->fetchAssoc($result)) {
			$dbusers[] = new DbUser($line["id"], $line["name"], $line["password"], $line["is_private"], $line["email"], $line["peer"], $line["is_admin"]);			
		}
		return $dbusers;
	}
	
	/**
	 * Remove user from database
	 * @param integer $userid the id of the user to be removed
	 * @throws DbException if database problem
	 */
	public static function removeDbUser($id) {
	 	Logger::write("Entering DbUser::removeDbUser", 1);	
		$dbconn = Db::getInstance();
		$query = "DELETE FROM users WHERE id = $id";
		$dbconn->update($query);
	}
	
	/**
	 * Get authenticated user. Returns the user if username and password match.
	 * Otherwise a DbUserException is thrown.
	 * @param string $username the username of the user
	 * @param string $password the password of the user
	 * @return DbUser the user as a DbUser object if the authentication was
	 * successful
	 * @throws DbUserException if the user could not be authenticated
	 * @throws DbException if database problem
	 */
	public static function getAuthUser($username, $password) {
	 	Logger::write("Entering DbUser::getAuthUser", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT id ".
				 "FROM users ".
				 "WHERE name = '$username' AND password = '$password'";
		$result = $dbconn->query($query);
		if($dbconn->numRows($result) == 0)
			throw new DbUserException("DbUser::getAuthUser", "Username and password does not match");
		$line = $dbconn->fetchAssoc($result);
		return DbUser::getDbUser($line["id"]);
	}
}
?>
