<?php
require_once("exceptions/ImageException.php");

/**
 * PNGImage implements Image and represents a PNG image.
 */
class PNGImage extends Image {

	/**
	 * Constructor for PNG images.
	 * @param string $filepath the absolute path and filename to the new image
	 * @throws ImageResource if the image resource could not be created
	 */
	function __construct($filepath) {
		Logger::write("Entering PNGImage::_construct", 1);
		parent::__construct($filepath);
		$this->createImageResource();
	}
	
	/**
	 * Output this image to the browser or to a file
	 * @param string $filepath absolute path and filename if image should be
	 * outputted to a file (optional). Default is output to browser
	 * @param integer $quality the quality of the outputted image if the format
	 * supports it. Otherwise it is ignored (optional). Default value is maximum
	 * quality
	 * @throws ImageException if the image resource could not be created,
	 * outputted or destroyed
	 */
	public function outputImage($filepath = '', $quality = 100) {
		Logger::write("Entering PNGImage::outputImage", 1);
		if(!(bool)$filepath) {
			header("Content-type: image/png");
			if(!imagepng($this->resource))
				throw new ImageException("PNGImage::outputImage", "Could not output the image");
		} else {
			if(!imagepng($this->resource, $filepath))
				throw new ImageException("PNGImage::outputImage", "Could not output the image to file");
		}
	}

	/**
	 * Resize this image 
	 * @param integer $width the new width of the resized image
	 * @param integer $height the new height of the resized image
	 * @throws ImageException if the resized image resource could not be
	 * created, outputted or destroyed
	 */
	public function resizeImage($width, $height) {
		Logger::write("Entering PNGImage::resizeImage", 1);
		if(!($image = imagecreatetruecolor($width, $height)))
			throw new ImageException("PNGImage::outputResizedImage", "Could not create new image resource");
		if(!imagecopyresampled($image, $this->resource, 0, 0, 0, 0, $width, $height, $this->width, $this->height))
			throw new ImageException("PNGImage::outputResizedImage", "Could not copy image");
		if(!imagedestroy($this->resource))
			throw new ImageException("PNGImage::outputResizedImage", "Could not destroy resized image resource");
		$this->width = $width;
		$this->height = $height;
		$this->resource = $image;
	}

	/**
	 * Create a new image resource of this image if it has previously been
	 * destroyed. If an image resource already exists it will be destroyed and
	 * recreated from file pointed to by this object
	 * @throws ImageException if the mage resource could not be created
	 */
	public function createImageResource() {
		Logger::write("Entering PNGImage::createImageResource", 1);
		if(!function_exists(imagecreatefrompng))
			throw new ImageException("PNGImage::createImageResource", "PNG support is not installed on this server");
		if($this->resource != null)
			$this->destroyImageResource();
		$resource = @imagecreatefrompng($this->filepath);
		if(!$resource)
			throw new ImageException("PNGImage::createImageResource", "Could not create image resource from '".$this->filepath."'");
		$this->resource = $resource;	
	}
}
?>